import { Ref, InjectionKey } from 'vue';
import { TurboQueryOptions, TurboQuery, TurboMutateValue } from 'turbo-query';

interface TurboVueOptions extends TurboQueryOptions {
    /**
     * A default turbo query instance to use if any.
     */
    turbo?: TurboQuery;
    immediate?: boolean;
}
interface TurboVueResourceActions<T> {
    /**
     * Refetches the current key.
     */
    refetch(opts?: TurboQueryOptions): Promise<T | undefined>;
    /**
     * Mutates the current key.
     */
    mutate(value: TurboMutateValue<T>): void;
    /**
     * Usubscribes from the current key changes.
     */
    unsubscribe(): void;
    /**
     * Forgets the current key from the cache.
     */
    forget(): void;
    /**
     * Aborts the current key's request if any.
     */
    abort(reason?: any): void;
    /**
     * Determines if an error occured.
     */
    error: Readonly<Ref<unknown>>;
    /**
     * Determines if it's refetching in the background.
     */
    isRefetching: Readonly<Ref<boolean>>;
}
declare type TurboVueResource<T> = [
    /**
     * The resulting resource.
     */
    Readonly<Ref<T | undefined>>,
    /**
     * Available actions on that resource.
     */
    TurboVueResourceActions<T>
];
/**
 * Determines how a Vue key looks like.
 */
declare type TurboVueKey = () => string | false | null;

declare const injectionKey: InjectionKey<TurboVueOptions>;
declare function injectQueryOptions(value?: TurboVueOptions): TurboVueOptions | undefined;
/**
 * Creates a new query resource with the given key and options
 * for suspense usage.
 */
declare function createQueryResource<T = any>(key: TurboVueKey, options?: TurboVueOptions): Promise<TurboVueResource<T>>;
/**
 * Creates a new query resource with the given key and options.
 */
declare function useQuery<T = any>(key: TurboVueKey, options?: TurboVueOptions): TurboVueResource<T>;

export { TurboVueKey, TurboVueOptions, TurboVueResource, TurboVueResourceActions, createQueryResource, injectQueryOptions, injectionKey, useQuery };
